<?php
/**
 * Gist importer class. Takes a GitHub Gist URL and returns content from that URL.
 *
 * @package    WordPress
 * @subpackage ff-block-gist-embed
 * @author     FrisFruitig <info@frisfruitig.com>
 * @license    https://www.gnu.org/licenses/gpl-2.0.html GPLv2
 * @link       https://www.frisfruitig.com
 * @since      5.0.3
 * @version    1.0.2
 */

namespace FrisFruitig;

// Require the parent class file.
require 'class-fileimporter.php';

/**
 * FileImporter subclass that handles GitHub Gists files
 */
class GistImporter extends FileImporter {

	/**
	 * Files in Gist with the name, URL, content and raw content per file.
	 *
	 * @var Array
	 */
	protected $files;


	/**
	 * Full content of the Gist embed.
	 * (Stored because we can use regular expressions on the content to split per file).
	 *
	 * @var String
	 */
	protected $full_content;


	/**
	 * Raw content per file.
	 *
	 * @var Array
	 */
	protected $raw_content;


	/**
	 * Gist stylesheet URL.
	 *
	 * @var String
	 */
	protected $stylesheet;


	/**
	 * Constructor.
	 *
	 * @param String $url  Validated Gist URL of the file to import.
	 */
	public function __construct( $url ) {
		parent::__construct( $this->validate_url( $url ) );
		$this->full_content = stripcslashes( parent::set_content( $this->url )['body'] );
		$this->content      = $this->set_content();
		$this->raw_content  = $this->set_raw_content();
		$this->files        = $this->set_files();
		$this->stylesheet   = $this->set_stylesheet();
	}


	/**
	 * Validate the Gist URL.
	 *
	 * @param String $url   URL provided in the constructor function.
	 * @return Bool|String  False if invalid URL, else validated URL.
	 */
	private function validate_url( $url ) {
		$parsed_url    = wp_parse_url( $url );
		$validated_url = $parsed_url['scheme'] . '://' . $parsed_url['host'] . $parsed_url['path'];
		// Append `.js` to URL path if it is not provided.
		if ( '.js' !== substr( $validated_url, -3 ) ) {
			$validated_url .= '.js';
		}
		// Append file query parameter if provided.
		if ( isset( $parsed_url['query'] ) ) {
			parse_str( $parsed_url['query'], $query );
			$validated_url .= '?file=' . $query['file'];
		}
		return $validated_url;
	}


	/**
	 * Store the content of each file related to the specified Gist.
	 *
	 * @return Array      List with the content of each file in the Gist.
	 */
	protected function set_content() {
		// Get each <div class="gist-file">...</div>, until either:
		// 1. the next occurance of <div class="gist-file">, or
		// 2. the last closing </div> (followed by a newline character) from the content.
		$regex = '/<div class=\"gist-file\">.+?(?=<div class=\"gist-file\">|<\/div>\n\')/ms';
		preg_match_all( $regex, $this->full_content, $matches );
		return $matches[0];
	}


	/**
	 * Store a list of filenames with the corresponding URL and content for each file related to the Gist.
	 */
	private function set_files() {
		// Get meta values and content for the files in the Gist.
		$meta  = $this->get_files_meta();
		$files = array();
		// Value to map indexes from $meta and $content array.
		$i = 0;
		foreach ( $meta as $filename => $url ) {
			// Push each descriptive array to the $files array and increment index counter.
			$file = array(
				'name'        => $filename,
				'url'         => $url,
				'content'     => $this->content[ $i ],
				'raw_content' => $this->raw_content[ $i ],
			);
			array_push( $files, $file );
			$i++;
		}
		return $files;
	}


	/**
	 * Get a list of filenames with the corresponding URL and content for each file related to the Gist.
	 *
	 * @return Array  Gist files.
	 */
	public function get_files() {
		return $this->files;
	}


	/**
	 * Get a list of filenames and the corresponding URL for each file related to the Gist.
	 *
	 * @return Array  List of the filenames and URLs.
	 */
	private function get_files_meta() {
		// Get each anchor that references a file, and capture both the URL [1] and the filename [2].
		$regex = '/<a href=\"([\d\w\:\/\.\-]*\#file-[\d\w-]*)\">(.+?)<\/a>/ms';
		preg_match_all( $regex, $this->full_content, $matches );
		$list = array_combine( $matches[2], $matches[1] );
		return $list;
	}


	/**
	 * Set the raw content of each file related to the specified Gist.
	 *
	 * @return Array      List with the raw content of each file in the Gist.
	 */
	private function set_raw_content() {
		// Get each <div class="gist-file">, until the next occurance of <div class="gist-meta">.
		$regex = '/<div class=\"gist-file\">.+?(?=<div class=\"gist-meta\">)/ms';
		preg_match_all( $regex, $this->full_content, $matches );
		// Strip all tags from string and decode what is left.
		return array_map( function( $match ) {
			return htmlspecialchars_decode( wp_strip_all_tags( $match ) );
		}, $matches[0] );
	}


	/**
	 * Get the raw content of each file related to the specified Gist.
	 *
	 * @return Array      List with the raw content of each file in the Gist.
	 */
	public function get_raw_content() {
		return $this->raw_content;
	}


	/**
	 * Get a list of filenames of each file in the Gist.
	 *
	 * @return Array  Filenames.
	 */
	public function get_filenames() {
		// Store $this->files in a temporary array, and remove 'url' and 'content' keys from that array.
		$filenames = array();
		foreach ( $this->files as $index => $file ) {
			array_push( $filenames, array( 'name' => $file['name'] ) );
		}
		return $filenames;
	}


	/**
	 * Set the location for the CSS file with style definitions for embedded Gists.
	 *
	 * @return String  Stylesheet URL.
	 */
	private function set_stylesheet() {
		// Get href value for the stylesheet in the document and return only the URL [1] instead of the full match.
		// phpcs:ignore WordPress.WP.EnqueuedResources.NonEnqueuedStylesheet
		$regex = '/<link rel=\"stylesheet\" href=\"([\d\w\:\/\.-]*)\">/ms';
		preg_match( $regex, $this->full_content, $match );
		return $match[1];
	}

	/**
	 * Get the location for the CSS file.
	 *
	 * @return String  Stylesheet URL.
	 */
	public function get_stylesheet() {
		return $this->stylesheet;
	}

}
